/*!
 * \file 
 * \brief Pollution Click example
 * 
 * # Description
 * Pollution click carries the VOC gas sensor and has high sensitivity to organic gases 
 * such as methanal (also known as formaldehyde), benzene, alcohol, toluene, etc.
 *
 * The demo application is composed of two sections :
 * 
 * ## Application Init 
 * Application Init performs Logger and Click initialization.
 * 
 * ## Application Task  
 * This is an example which demonstrates the usage of Pollution Click board.
 * Pollution Click reads ADC value, load voltage from ADC value, and reads corrected 
 * resistance of the sensor where results are being sent to the UART terminal 
 * where you can track changes.
 * 
 * \author Mihajlo Djordjevic
 *
 */
// ------------------------------------------------------------------- INCLUDES

#include "board.h"
#include "log.h"
#include "pollution.h"

// ------------------------------------------------------------------ VARIABLES

static pollution_t pollution;
static log_t logger;

// ------------------------------------------------------ APPLICATION FUNCTIONS

void application_init ( void )
{
    log_cfg_t log_cfg;
    pollution_cfg_t cfg;

    //  Logger initialization.

    LOG_MAP_USB_UART( log_cfg );
    log_cfg.baud = 9600;
    log_cfg.level = LOG_LEVEL_DEBUG;
    log_init( &logger, &log_cfg );
    log_printf( &logger, "---------- Application Init ---------\r\n" );
    Delay_ms( 1000 );

    //  Click initialization.

    pollution_cfg_setup( &cfg );
    POLLUTION_MAP_MIKROBUS( cfg, MIKROBUS_1 );
    pollution_init( &pollution, &cfg );

    log_printf( &logger, "-------------------------------------\r\n" );
    log_printf( &logger, "---------- Pollution Click ----------\r\n" );
    log_printf( &logger, "-------------------------------------\r\n" );
    Delay_ms( 500 );

    pollution_default_cfg( &pollution );

    log_printf( &logger, "---------- Application Init Done ----\r\n" );
    log_printf( &logger, "-------------------------------------\r\n" );
    log_printf( &logger, "---------- Click Calibration --------\r\n" );
    Delay_ms( 1000 );

    //  Click calibration.

    for ( uint8_t cnt = 0; cnt < 20; cnt++ )
    {
        pollution_update_calibrated_resistance( &pollution );
        Delay_ms( 100 );
    }

    pollution_update_corrected_resistance( &pollution );

    log_printf( &logger, "---------- Calibration Done ---------\r\n\n" );
}

void application_task ( void )
{
    float meas_resistance = pollution_update_measured_resistance( &pollution );
    float meas_ppm = pollution_ratio_to_ppm( &pollution, meas_resistance );

    log_printf( &logger, " Pollution [ppm] : %.2f\r\n", meas_ppm );
    log_printf( &logger, "-------------------------------------\r\n" );
    Delay_ms( 1000 );
}

void main ( void )
{
    application_init( );

    for ( ; ; )
    {
        application_task( );
    }
}

// ------------------------------------------------------------------------ END
