/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright© 2020 MikroElektronika d.o.o.
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be 
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE 
 * OR OTHER DEALINGS IN THE SOFTWARE. 
 */

/*!
 * \file
 *
 */

#include "pollution.h"
#include "math.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define HYDROGEN_MAX_R_LOAD          9950
#define POLLUTION_VCC                4.97
#define POLLUTION_C1                 0.8639656653028277     /**< Constant for calculating ppm (I segment). */
#define POLLUTION_C2                 1.1496082990709344     /**< Constant for calculating ppm (II segment). */
#define POLLUTION_C3                 1.5489389323538676     /**< Constant for calculating ppm (III segment). */
#define POLLUTION_M1                 -0.30361905759780833   /**< Constant for calculating ppm (I segment). */
#define POLLUTION_M2                 -0.4694852833012203    /**< Constant for calculating ppm (II segment). */
#define POLLUTION_M3                 -0.5989696888059545    /**< Constant for calculating ppm (III segment). */

// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS 

static void dev_comm_delay ( void );

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void pollution_cfg_setup ( pollution_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->an           = HAL_PIN_NC;

    // Additional gpio pins

    cfg->ena          = HAL_PIN_NC;

    // ADC characteristics

    cfg->resolution   = ANALOG_IN_RESOLUTION_10_BIT;
    cfg->vref         = 3.3;
}

POLLUTION_RETVAL pollution_init ( pollution_t *ctx, pollution_cfg_t *cfg )
{
    analog_in_config_t adc_cfg;

    analog_in_configure_default( &adc_cfg );

    adc_cfg.input_pin = cfg->an;

    if ( analog_in_open( &ctx->adc, &adc_cfg ) == ADC_ERROR )
    {
        return POLLUTION_INIT_ERROR;
    }

    analog_in_set_vref_value( &ctx->adc, cfg->vref );
    analog_in_set_resolution( &ctx->adc, cfg->resolution );

    // Output pins 

    digital_out_init( &ctx->ena, cfg->ena );

    ctx->calib_res   = 0;
    ctx->correct_res = 0;

    return POLLUTION_OK;
}

void pollution_default_cfg ( pollution_t *ctx )
{
    // Reset the WSP2110 sensor

    pollution_reset( ctx );
}

uint16_t pollution_generic_read ( pollution_t *ctx )
{
    uint16_t rx_data;

    analog_in_read( &ctx->adc, &rx_data );

    return rx_data;
}

void pollution_enable ( pollution_t *ctx )
{
    digital_out_high( &ctx->ena );
}

void pollution_disable ( pollution_t *ctx )
{
    digital_out_low( &ctx->ena );
}

void pollution_reset ( pollution_t *ctx )
{
    digital_out_high( &ctx->ena );
    dev_comm_delay( );

    digital_out_low( &ctx->ena );
    dev_comm_delay( );

    digital_out_high( &ctx->ena );
    dev_comm_delay( );
}

float pollution_measure_load_voltage ( pollution_t *ctx )
{
    float voltage;

    analog_in_read_voltage( &ctx->adc, &voltage );

    return voltage;
}

float pollution_update_calibrated_resistance ( pollution_t *ctx )
{
    float v_load;

    v_load = pollution_measure_load_voltage( ctx );

    ctx->calib_res  = POLLUTION_VCC;
    ctx->calib_res /= v_load;
    ctx->calib_res -= 1;
    ctx->calib_res *= HYDROGEN_MAX_R_LOAD;

    return ctx->calib_res;
}

float pollution_update_corrected_resistance ( pollution_t *ctx )
{
    float v_load;

    v_load = pollution_measure_load_voltage( ctx );

    ctx->correct_res  = v_load;
    ctx->correct_res /= POLLUTION_VCC - v_load;
    ctx->correct_res *= ctx->calib_res;

    return ctx->correct_res;
}

float pollution_update_measured_resistance ( pollution_t *ctx )
{
    float result;
    float v_load;

    v_load = pollution_measure_load_voltage( ctx );

    result  = POLLUTION_VCC;
    result /= v_load;
    result -= 1;
    result *= ctx->correct_res;

    return result;
}

float pollution_ratio_to_ppm ( pollution_t *ctx, float meas_res )
{
    float ratio = meas_res / ctx->calib_res > 1.0 ? 1.0 : meas_res / ctx->calib_res;

    if ( ( 0.84 > ratio ) && ( ratio >= 0.53 ) )        // I segment
        return pow( ( ratio / POLLUTION_C1 ), ( 1 / POLLUTION_M1 ) );

    else if ( ( 0.53 > ratio ) && ( ratio >= 0.39 ) )   // II segment
        return pow( ( ratio / POLLUTION_C2 ), ( 1 / POLLUTION_M2 ) );

    else if ( ( 0.39 > ratio ) && ( ratio >= 0.148 ) )  // III segment
        return pow( ( ratio / POLLUTION_C3 ), ( 1 / POLLUTION_M3 ) );

    else if ( ratio < 0.148 )
        return 50;

    return 1;
}

// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

static void dev_comm_delay ( void )
{
    Delay_100ms( );
}

// ------------------------------------------------------------------------ END
